<?php

namespace Drupal\graphql_webform\Plugin\GraphQL\DataProducer;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\GraphQL\Execution\FieldContext;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Drupal\graphql_core_schema\GraphQL\Buffers\SubRequestBuffer;
use Drupal\webform\WebformInterface;
use GraphQL\Deferred;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Render a webform and return the render array.
 *
 * @DataProducer(
 *   id = "webform_render",
 *   name = @Translation("Form Render"),
 *   description = @Translation("Returns the render array of a form."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("Render Array"),
 *     required = FALSE
 *   ),
 *   consumes = {
 *     "webform" = @ContextDefinition("entity:webform",
 *       label = @Translation("Webform")
 *     )
 *   }
 * )
 */
class WebformRender extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The subrequest buffer.
   */
  protected SubRequestBuffer $subRequestBuffer;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $pluginId,
    $pluginDefinition
  ) {
    return new static(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $container->get('graphql_core_schema.buffer.subrequest')
    );
  }

  /**
   * NodeLogoTheme constructor.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $pluginId
   *   The plugin id.
   * @param mixed $pluginDefinition
   *   The plugin definition.
   * @param SubRequestBuffer $subRequestBuffer
   *   The sub-request buffer service.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    $pluginDefinition,
    SubRequestBuffer $subRequestBuffer
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->subRequestBuffer = $subRequestBuffer;
  }

  /**
   * Resolver.
   */
  public function resolve(WebformInterface $webform, FieldContext $field) {
    $sourceEntityType = $field->getContextValue('webform_source_entity_type');
    $sourceEntityId = $field->getContextValue('webform_source_entity_id');
    $url = $webform->toUrl();

    if ($sourceEntityType && $sourceEntityId) {
      $url->setOption('query', [
        'source_entity_type' => $sourceEntityType,
        'source_entity_id' => $sourceEntityId,
      ]);
    }
    $field->setContextValue('webform', $webform);

    $resolve = $this->subRequestBuffer->add($url, function () use ($webform) {
      return $webform->getSubmissionForm();
    });

    return new Deferred(function () use ($resolve) {
      return $resolve();
    });
  }

}
