<?php

declare(strict_types = 1);

namespace Drupal\graphql_webform\Plugin\GraphQL\DataProducer;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Drupal\webform\WebformInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Load the settings for a webform.
 *
 * @DataProducer(
 *   id = "webform_settings",
 *   name = @Translation("Webform Settings"),
 *   description = @Translation("Load the settings for a webform."),
 *   produces = @ContextDefinition("map",
 *     label = @Translation("Webform Settings"),
 *     required = FALSE
 *   ),
 *   consumes = {
 *     "webform" = @ContextDefinition("entity:webform",
 *       label = @Translation("Webform")
 *     )
 *   }
 * )
 */
class WebformSettings extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The Webform default configuration settings.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $pluginId,
    $pluginDefinition
  ) {
    return new static(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $container->get('config.factory')
    );
  }

  /**
   * WebformSettings constructor.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $pluginId
   *   The plugin id.
   * @param mixed $pluginDefinition
   *   The plugin definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The configuration object factory.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    $pluginDefinition,
    ConfigFactoryInterface $configFactory
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->config = $configFactory->get('webform.settings');
  }

  /**
   * Resolver.
   *
   * @param \Drupal\webform\Entity\Webform $webform
   *   The webform to get the settings for.
   */
  public function resolve(WebformInterface $webform): array {
    $globalSettings = $this->config->get('settings');
    $defaults = [];

    foreach ($globalSettings as $key => $setting) {
      if (str_starts_with($key, 'default_')) {
        $settingKey = substr($key, 8);
        $defaults[$settingKey] = $setting;
      }
    }
    $settings = $webform->getSettings();
    return array_merge($defaults, array_filter($settings));
  }

}
