<?php

namespace Drupal\graphql_webform\Plugin\GraphQL\DataProducer;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\graphql\GraphQL\Execution\FieldContext;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Drupal\webform\WebformMessageManagerInterface;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform\WebformTokenManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Return the webform submission confirmation.
 *
 * @DataProducer(
 *   id = "webform_submission_confirmation",
 *   name = @Translation("Webform Submission Confirmation"),
 *   description = @Translation("Returns the webform confirmation."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("Webform Items"),
 *     required = FALSE
 *   ),
 *   consumes = {
 *     "submission" = @ContextDefinition("entity:webform_submission",
 *       label = @Translation("Webform Submission")
 *     )
 *   }
 * )
 */
class WebformSubmissionConfirmation extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The webform message manager.
   *
   * @var \Drupal\webform\WebformMessageManagerInterface
   */
  protected WebformMessageManagerInterface $messageManager;

  /**
   * The webform token manager.
   *
   * @var \Drupal\webform\WebformTokenManagerInterface
   */
  protected WebformTokenManagerInterface $tokenManager;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $pluginId,
    $pluginDefinition
  ) {
    return new static(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $container->get('webform.message_manager'),
      $container->get('webform.token_manager'),
      $container->get('renderer')
    );
  }

  /**
   * WebformSubmissionConfirmation constructor.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $pluginId
   *   The plugin id.
   * @param mixed $pluginDefinition
   *   The plugin definition.
   * @param \Drupal\webform\WebformMessageManager $webformMessageManager
   *   The webform message manager.
   * @param \Drupal\webform\WebformTokenManagerInterface $webformTokenManager
   *   The webform token manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    $pluginDefinition,
    WebformMessageManagerInterface $webformMessageManager,
    WebformTokenManagerInterface $webformTokenManager,
    RendererInterface $renderer,
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
    $this->messageManager = $webformMessageManager;
    $this->tokenManager = $webformTokenManager;
    $this->renderer = $renderer;
  }

  /**
   * Resolver.
   */
  public function resolve(WebformSubmissionInterface $submission, FieldContext $field) {
    $renderContext = new RenderContext();
    $renderResult = $this->renderer->executeInRenderContext($renderContext, function () use ($submission) {
      $webform = $submission->getWebform();
      $sourceEntity = $submission->getSourceEntity();

      $this->messageManager->setSourceEntity($sourceEntity);
      $this->messageManager->setWebformSubmission($submission);
      $this->messageManager->setWebform($webform);

      $title = $webform->getSetting('confirmation_title') ?: (($sourceEntity) ? $sourceEntity->label() : $webform->label());
      $title = $this->tokenManager->replace($title, $submission ?: $webform);

      if (is_array($title)) {
        $title = $this->renderer->renderPlain($title);
      }

      $message = $this->messageManager->build(
        WebformMessageManagerInterface::SUBMISSION_CONFIRMATION_MESSAGE
      );
      if (is_array($message)) {
        $message = $this->renderer->renderPlain($message);
      }

      return [
        'title' => (string) $title,
        'message' => (string) $message,
        'submission' => $submission,
      ];
    });

    return $renderResult;
  }

}
