<?php

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\graphql\GraphQL\ResolverBuilder;
use Drupal\graphql\GraphQL\ResolverRegistryInterface;
use Drupal\graphql\Plugin\GraphQL\SchemaExtension\SdlSchemaExtensionPluginBase;
use Drupal\graphql_core_schema\CoreSchemaExtensionInterface;
use Drupal\graphql_core_schema\EntitySchemaHelper;
use Drupal\graphql_webform\WebformSchemaBuilder;
use Drupal\graphql_webform\Wrappers\WebformSubmissionResult;
use Drupal\graphql_webform\Wrappers\WebformSubmissionValidation;
use Drupal\webform\Utility\WebformOptionsHelper;
use Drupal\webform\WebformSubmissionInterface;

class WebformExtension extends SdlSchemaExtensionPluginBase implements ContainerFactoryPluginInterface, CoreSchemaExtensionInterface {

  /**
   * {@inheritdoc}
   */
  public function getBaseDefinition() {
    $definitions = $this->elementManager->getDefinitions();
    $schemaBuilder = new WebformSchemaBuilder();
    foreach ($definitions as $id => $definition) {
      /** @var \Drupal\webform\Plugin\WebformElementBase $instance */
      $instance = $this->elementManager->createInstance($id, []);
      $label = (string) $definition['label'];
      $description = (string) $definition['description'];
      $schemaBuilder->generateElementType($id, $label, $description, $instance);
    }
    $schemaBuilder->generateElementType('webform_multiple', 'Multiple', '');
    $schemaBuilder->generateElementType('unexposed', 'Unexposed', 'Unexposed webform element.');
    $schemaBuilder->generateElementType('submit', 'Submit', 'Submit button.');

    return $schemaBuilder->getGeneratedSchema() . "\n\n" . $this->loadDefinitionFile('WebformSubmission');
  }

  /**
   * {@inheritdoc}
   */
  public function registerResolvers(ResolverRegistryInterface $registry): void {
    $builder = new ResolverBuilder();

    $registry->addTypeResolver('WebformElement', function ($element) {
      $type = $element['#type'];
      if ($type) {
        if (in_array($type, $this->elementIds)) {
          return EntitySchemaHelper::toPascalCase(['WebformElement', $type]);
        }
        elseif ($type === 'webform_multiple') {
          return 'WebformElementWebformMultiple';
        }
        elseif ($type === 'submit') {
          return 'WebformElementSubmit';
        }
      }

      return 'WebformElementUnexposed';
    });

    $registry->addTypeResolver('WebformForm', function ($element) {
      return 'WebformForm';
    });

    $registry->addFieldResolver('Query', 'webformConfirmation',
      $builder->compose(
        $builder->produce('webform_load_submission')
          ->map('id', $builder->fromArgument('id'))
          ->map('token', $builder->fromArgument('token')),
        $builder->produce('webform_submission_confirmation')
          ->map('submission', $builder->fromParent())
      )
    );

    $registry->addFieldResolver('WebformFormItem', 'key', $builder->compose(
      $builder->callback(function ($value) {
        $key = $value['#webform_key'] ?? '';
        return $key;
      })
    ));

    $registry->addFieldResolver('WebformFormItem', 'name', $builder->compose(
      $builder->callback(function ($value) {
        $name = $value['#name'] ?? '';
        // When an element can be multiple, we remove the nested name of the
        // input. That way multiple elements with the same name can be
        // submitted and will be parsed as multiple entries.
        return str_replace('[items][0][_item_]', '[0]', $name);
      })
    ));

    $registry->addFieldResolver('WebformFormItem', 'element', $builder->compose(
      $builder->callback(function ($value) {
        return $value;
      })
    ));

    $typeFields = [
      'WebformFormItem' => [
        'title' => ['string'],
        'help' => ['string'],
        'helpTitle' => ['string'],
        'helpDisplay' => ['string'],
        'more' => ['string'],
        'type' => ['string'],
        'moreTitle' => ['string'],
        'titleDisplay' => ['string'],
        'defaultValue' => ['string'],
        'description' => ['string'],
        'descriptionDisplay' => ['string'],
        'disabled' => ['boolean'],
        'readonly' => ['boolean'],
        'fieldPrefix' => ['string'],
        'fieldSuffix' => ['string'],
        'hasGarbageValue' => ['boolean'],
        'input' => ['boolean'],
        'processed' => ['boolean'],
        'required' => ['boolean'],
        'requiredError' => ['string'],
        'tree' => ['boolean'],
        'flex' => ['int', 1],
      ],
      'WebformElement' => [
        'type' => ['string'],
      ],
      'WebformElementDateBase' => [
        'dateDateMin' => ['string'],
        'dateDateMax' => ['string'],
        'dateDays' => ['array'],
      ],
      'WebformElementWebformMultiple' => [
        'minItems' => ['int'],
        'addMore' => ['boolean'],
        'addMoreItems' => ['int'],
        'addMoreButtonLabel' => ['string'],
        'addMoreInput' => ['boolean'],
        'addMoreInputLabel' => ['string'],
        'sorting' => ['boolean'],
        'operations' => ['boolean'],
        'add' => ['boolean'],
        'remove' => ['boolean'],
      ],
      'WebformElementOptionsBase' => [
        'optionsRandomize' => ['boolean'],
      ],
      'WebformElementTextfield' => [
        'inputMask' => ['string'],
      ],
      'WebformElementWebformFlexbox' => [
        'alignItems' => ['string'],
      ],
      'WebformElementWebformSection' => [
        'titleTag' => ['string'],
      ],
      'WebformElementNumber' => [
        'min' => ['int'],
        'max' => ['int'],
        'step' => ['int'],
      ],
      'WebformElementTextBase' => [
        'readonly' => ['boolean'],
        'size' => ['int'],
        'minlength' => ['int'],
        'maxlength' => ['int'],
        'placeholder' => ['string'],
        'autocomplete' => ['string'],
        'pattern' => ['string'],
        'patternError' => ['string'],
      ],
    ];

    foreach ($typeFields as $interface => $fields) {
      foreach ($fields as $field => $config) {
        $registry->addFieldResolver(
          $interface,
          $field,
          $builder->produce('webform_element_property')
            ->map('element', $builder->fromParent())
            ->map('property', $builder->fromValue($field))
            ->map('type', $builder->fromValue($config[0]))
            ->map('default', $builder->fromValue($config[1] ?? NULL))
        );
      }
    }

    // Resolver for the address fields.
    $registry->addFieldResolver('WebformElementWebformAddress', 'items', $builder->compose(
      $builder->callback(function ($value) {
        $elementKeys = array_keys($value['#webform_composite_elements']);
        $elements = [];

        foreach ($elementKeys as $key) {
          if (!empty($value[$key])) {
            $elements[] = $value[$key];
          }
        }
        return $elements;
      })
    ));

    // Resolver for container elements.
    $registry->addFieldResolver('WebformElementContainerBase', 'items', $builder->compose(
      $builder->produce('webform_items')
        ->map('form', $builder->fromParent())
        ->map('webform', $builder->fromContext('webform'))
    ));

    // Resolver for the webform_multiple element.
    $registry->addFieldResolver('WebformElementWebformMultiple', 'items', $builder->compose(
      $builder->callback(function ($value) {
        $element = $value['items'][0]['_item_'] ?? NULL;
        return $element ? [$element] : [];
      })
    ));

    // Resolve the options for checkboxes, radios, etc.
    $registry->addFieldResolver('WebformElementOptionsBase', 'options', $builder->compose(
      $builder->callback(function ($value) {
        $optionsRaw = $value['#options'] ?? [];
        $options = [];

        foreach ($optionsRaw as $value => $data) {
          $parts = WebformOptionsHelper::splitOption($data);
          $options[] = [
            'value' => $value,
            'label' => $parts[0],
            'description' => $parts[1] ?? '',
          ];
        }
        return $options;
      })
    ));

    $this->registerElementOtherResolvers($registry, $builder);
    $this->registerStateFields($registry, $builder);
    $this->addWebformFields($registry, $builder);
    $this->addMutationFields($registry, $builder);

    // Resolver for the webform_multiple element.
    $registry->addFieldResolver('WebformElementManagedFileBase', 'maxFilesize',
      $builder->callback(function ($value) {
        $size = $value['#upload_validators']['file_validate_size'] ?? NULL;
        if ($size && is_array($size)) {
          return $size[0] ?? NULL;
        }
        return $size;
      })
    );

    $registry->addFieldResolver('WebformElementManagedFileBase', 'fileExtensions',
      $builder->callback(function ($value) {
        return $value['#upload_validators']['file_validate_size'] ?? NULL;
      })
    );

    $registry->addFieldResolver('WebformElementManagedFileBase', 'cardinality',
      $builder->callback(function ($value) {
        /** @var null|int|bool $multiple */
        $multiple = $value['#graphql_element_decoded']['#multiple'] ?? NULL;

        if (is_int($multiple)) {
          // Explicit amount of allowed items.
          return $multiple;
        }
        elseif ($multiple === TRUE) {
          // Unlimited.
          return -1;
        }

        // Only one.
        return 1;
      })
    );

    $registry->addFieldResolver('WebformElementSubmit', 'label',
      $builder->callback(function ($value) {
        return $value['#value'] ?? NULL;
      })
    );
  }

  /**
   * Register the basic webform fields.
   *
   * @param \Drupal\graphql\GraphQL\ResolverRegistryInterface $registry
   *   The resolver registry.
   * @param \Drupal\graphql\GraphQL\ResolverBuilder $builder
   *   The resolver builder.
   */
  protected function addWebformFields(ResolverRegistryInterface $registry, ResolverBuilder $builder): void {
    $registry->addFieldResolver(
      'Webform',
      'settings',
      $builder
        ->produce('webform_settings')
        ->map('webform', $builder->fromParent())
    );

    // Resolver for the main elements field on the webform.
    $registry->addFieldResolver(
      'WebformForm',
      'items',
      $builder
        ->produce('webform_items')
        ->map('form', $builder->fromParent())
        ->map('webform', $builder->fromContext('webform'))
    );

    $registry->addFieldResolver(
      'WebformForm',
      'sourceEntity',
      $builder->produce('entity_load')
        ->map('type', $builder->fromContext('webform_source_entity_type'))
        ->map('id', $builder->fromContext('webform_source_entity_id'))
    );

    // Resolver for the form.
    $registry->addFieldResolver(
      'Webform',
      'form',
      $builder
        ->produce('webform_render')
        ->map('webform', $builder->fromParent())
    );
  }

  /**
   * Register the webform element state fields.
   *
   * @param \Drupal\graphql\GraphQL\ResolverRegistryInterface $registry
   *   The resolver registry.
   * @param \Drupal\graphql\GraphQL\ResolverBuilderInterface $builder
   *   The resolver builder.
   */
  protected function addMutationFields(ResolverRegistryInterface $registry, ResolverBuilder $builder): void {
    // The webform submission mutation.
    $registry->addFieldResolver('Mutation', 'submitWebform', $builder->compose(
      $builder->produce('webform_submit')
        ->map('id', $builder->fromArgument('id'))
        ->map('elements', $builder->fromArgument('elements'))
        ->map('files', $builder->fromArgument('files'))
        ->map('sourceEntityId', $builder->fromArgument('sourceEntityId'))
        ->map('sourceEntityType', $builder->fromArgument('sourceEntityType'))
    ));

    $registry->addFieldResolver('WebformSubmissionResult', 'submission',
      $builder->callback(function (WebformSubmissionResult $result) {
        return $result->getSubmission();
      })
    );

    $registry->addFieldResolver('WebformSubmissionResult', 'submissionToken',
      $builder->callback(function (WebformSubmissionResult $result) {
        $submission = $result->getSubmission();
        return $submission ? $submission->getToken() : NULL;
      })
    );

    $registry->addFieldResolver('WebformSubmissionResult', 'submissionId',
      $builder->callback(function (WebformSubmissionResult $result) {
        $submission = $result->getSubmission();
        return $submission ? $submission->id() : NULL;
      })
    );

    $registry->addFieldResolver('WebformSubmission', 'tokenUrl',
      $builder->callback(function (WebformSubmissionInterface $submission) {
        return $submission->getTokenUrl('view');
      })
    );

    $registry->addFieldResolver('WebformSubmissionResult', 'errors',
      $builder->callback(function (WebformSubmissionResult $result) {
        return $result->getErrors();
      })
    );

    $registry->addFieldResolver('WebformSubmissionResult', 'validations',
      $builder->callback(function (WebformSubmissionResult $result) {
        return $result->getValidations();
      })
    );

    $registry->addFieldResolver('WebformSubmissionResult', 'confirmation',
      $builder
        ->produce('webform_submission_confirmation')
        ->map('submission', $builder->fromParent()
      )
    );

    $registry->addFieldResolver('WebformSubmissionValidation', 'messages',
      $builder->callback(function (WebformSubmissionValidation $result) {
        return $result->getMessages();
      })
    );

    $registry->addFieldResolver('WebformSubmissionValidation', 'element',
      $builder->callback(function (WebformSubmissionValidation $result) {
        return $result->getElementId();
      })
    );
  }

  /**
   * Register the "other" webform elements.
   *
   * @param \Drupal\graphql\GraphQL\ResolverRegistryInterface $registry
   *   The resolver registry.
   * @param \Drupal\graphql\GraphQL\ResolverBuilderInterface $builder
   *   The resolver builder.
   */
  protected function registerElementOtherResolvers(ResolverRegistryInterface $registry, ResolverBuilder $builder): void {
    // Add the other field, which is a nested element.
    $registry->addFieldResolver('WebformElementOtherBase', 'other', $builder->compose(
      $builder->callback(function ($value) {
        return $value['other'];
      })
    ));

    $registry->addFieldResolver('WebformElementWebformRadiosOther', 'radios', $builder->compose(
      $builder->callback(function ($value) {
        return $value['radios'];
      })
    ));

    $registry->addFieldResolver('WebformElementWebformSelectOther', 'select', $builder->compose(
      $builder->callback(function ($value) {
        return $value['select'];
      })
    ));

    $registry->addFieldResolver('WebformElementWebformSelectOther', 'checkboxes', $builder->compose(
      $builder->callback(function ($value) {
        return $value['checkboxes'];
      })
    ));
  }

  /**
   * Register the webform element state fields.
   *
   * @param \Drupal\graphql\GraphQL\ResolverRegistryInterface $registry
   *   The resolver registry.
   * @param \Drupal\graphql\GraphQL\ResolverBuilderInterface $builder
   *   The resolver builder.
   */
  protected function registerStateFields(ResolverRegistryInterface $registry, ResolverBuilder $builder): void {
    // Add the other field, which is a nested element.
    $registry->addFieldResolver('WebformFormItem', 'states', $builder->compose(
      $builder->callback(function ($value) {
        return $value['#states'] ?? NULL;
      })
    ));

    // Add the other field, which is a nested element.
    foreach (WebformSchemaBuilder::WEBFORM_STATES as $state) {
      $registry->addFieldResolver(
        'WebformElementStates',
        $state,
        $builder->produce('webform_element_state')->map('element', $builder->fromParent())->map('state', $builder->fromValue($state))
      );
    }

    $registry->addFieldResolver('WebformElementState', 'conditions', $builder->compose(
      $builder->callback(function ($value) {
        return $value['conditions'];
      })
    ));

    $registry->addFieldResolver('WebformElementState', 'logic', $builder->compose(
      $builder->callback(function ($value) {
        return $value['logic'];
      })
    ));

    $registry->addFieldResolver('WebformElementStateCondition', 'field', $builder->compose(
      $builder->callback(function ($value) {
        $selector = $value['selector'] ?? '';
        $matches = [];
        preg_match('/:input\[name="(.*?)["[]/', $selector, $matches);
        return $matches[1] ?? NULL;
      })
    ));

    $registry->addFieldResolver('WebformElementStateCondition', 'fieldValue', $builder->compose(
      $builder->callback(function ($value) {
        $condition_value = $value['value'] ?? '';
        $selector = $value['selector'] ?? '';
        $matches = [];
        preg_match('/:input\[name=".*\[(.*)]"/', $selector, $matches);

        return $matches[1] ?? $condition_value;
      })
    ));

    $registry->addFieldResolver('WebformElementStateCondition', 'selector', $builder->compose(
      $builder->callback(function ($value) {
        return $value['selector'] ?? '';
      })
    ));

    $registry->addFieldResolver('WebformElementStateCondition', 'trigger', $builder->compose(
      $builder->callback(function ($value) {
        $trigger = $value['trigger'] ?? '';
        return WebformSchemaBuilder::WEBFORM_TRIGGERS[$trigger];
      })
    ));

    $registry->addFieldResolver('WebformElementStateCondition', 'value', $builder->compose(
      $builder->callback(function ($value) {
        return $value['value'] ?? '';
      })
    ));

    $registry->addFieldResolver('Webform', 'logic', $builder->compose(
      $builder->callback(function ($value) {
        return $value['logic'];
      })
    ));
  }

}
