<?php

declare(strict_types=1);

namespace Drupal\graphql_webform\Model;

use Drupal\webform\WebformSubmissionInterface;

/**
 * Data object representing the result of a Webform submission.
 */
class WebformSubmissionResult {

  /**
   * The Webform submission entity, if the submission was successful.
   */
  protected ?WebformSubmissionInterface $submission = NULL;

  /**
   * A list of error messages that occurred during submission.
   *
   * @var string[]
   */
  protected array $errors = [];

  /**
   * A list of validation errors that occurred during submission.
   *
   * @var \Drupal\graphql_webform\Model\WebformSubmissionValidationError[]
   */
  protected array $validationErrors = [];

  /**
   * Sets the Webform submission.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $submission
   *   The Webform submission entity.
   *
   * @return \Drupal\graphql_webform\Model\WebformSubmissionResult
   *   The current class, for chaining.
   */
  public function setSubmission(WebformSubmissionInterface $submission): static {
    $this->submission = $submission;
    return $this;
  }

  /**
   * Returns the Webform submission entity.
   *
   * @return \Drupal\webform\WebformSubmissionInterface|null
   *   The Webform submission entity.
   */
  public function getSubmission(): ?WebformSubmissionInterface {
    return $this->submission;
  }

  /**
   * Adds an error message.
   *
   * These are intended for general errors. For validation errors, call
   * ::addValidationError() instead.
   *
   * @param string $error
   *   The error to add.
   *
   * @return \Drupal\graphql_webform\Model\WebformSubmissionResult
   *   The WebformSubmissionResult, for chaining.
   */
  public function addError(string $error): static {
    // The Webform module returns error messages intended to be shown in a web
    // page. Strip HTML tags to make them suitable for general use.
    $error = trim(strip_tags($error));
    $this->errors[] = $error;
    return $this;
  }

  /**
   * Returns the list of errors.
   *
   * @return string[]
   *   The errors.
   */
  public function getErrors(): array {
    return $this->errors;
  }

  /**
   * Returns the list of validation errors.
   *
   * @return \Drupal\graphql_webform\Model\WebformSubmissionValidationError[]
   *   The validation errors.
   */
  public function getValidationErrors(): array {
    return $this->validationErrors;
  }

  /**
   * Adds a validation error.
   *
   * @param \Drupal\graphql_webform\Model\WebformSubmissionValidationError $validation_error
   *   The validation error to add.
   */
  public function addValidationError(WebformSubmissionValidationError $validation_error): static {
    $this->validationErrors[] = $validation_error;
    return $this;
  }

  /**
   * Returns whether the submission is valid.
   *
   * @return bool
   *   TRUE if there are no errors or validation errors.
   */
  public function isValid(): bool {
    return empty($this->errors) && empty($this->validationErrors);
  }

}
