<?php

declare(strict_types=1);

namespace Drupal\graphql_webform\Plugin\GraphQL\DataProducer;

use Drupal\Core\Render\Element;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Drupal\webform\WebformInterface;

/**
 * Returns the Webform elements.
 *
 * @DataProducer(
 *   id = "webform_elements",
 *   name = @Translation("Webform elements"),
 *   description = @Translation("Returns the Webform elements."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("Webform elements"),
 *     required = FALSE
 *   ),
 *   consumes = {
 *     "webform" = @ContextDefinition("entity:webform",
 *       label = @Translation("Webform"),
 *       required = TRUE
 *     ),
 *     "parent" = @ContextDefinition("any",
 *       label = @Translation("Parent element"),
 *       required = FALSE
 *     )
 *   }
 * )
 */
class WebformElements extends DataProducerPluginBase {

  /**
   * Resolves the Webform elements.
   *
   * @param \Drupal\webform\WebformInterface $webform
   *   The full webform.
   * @param array|null $parent
   *   An optional parent element. If this is passed, the child elements will be
   *   resolved. If this is not passed, the top-level elements will be resolved.
   *
   * @return array
   *   An array of Webform render array elements.
   */
  public function resolve(WebformInterface $webform, ?array $parent = NULL): array {
    // Retrieve the form elements. First, try to get the elements from the
    // parent element. If this is not passed, try to get the elements from the
    // base Webform.
    $form = $parent ?? $webform->getSubmissionForm() ?? [];
    $original_elements = empty($form['elements']) ? $form : $form['elements'];

    // Get a list of "decoded" elements. These contain the element properties
    // and metadata as configured in the Webform.
    $decoded_elements = $webform->getElementsDecodedAndFlattened('view');

    // Loop over the original elements and store the "decoded" information in
    // them, so we can peruse it in other resolvers.
    $elements = [];
    foreach (Element::children($original_elements, TRUE) as $key) {
      $element = $original_elements[$key];
      if (!empty($element)) {
        $element['#graphql_element_decoded'] = $decoded_elements[$key] ?? NULL;
        $elements[] = $element;
      }
    }
    return $elements;
  }

}
