<?php

declare(strict_types=1);

namespace Drupal\graphql_webform\Plugin\GraphQL\DataProducer;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\graphql\GraphQL\Execution\FieldContext;
use Drupal\graphql\GraphQL\Utility\FileUpload;
use Drupal\graphql\Plugin\GraphQL\DataProducer\DataProducerPluginBase;
use Drupal\graphql_webform\Model\WebformSubmissionResult;
use Drupal\graphql_webform\Model\WebformSubmissionValidationError;
use Drupal\webform\Element\Webform;
use Drupal\webform\Plugin\WebformElementManagerInterface;
use Drupal\webform\WebformEntityStorageInterface;
use Drupal\webform\WebformInterface;
use Drupal\webform\WebformSubmissionForm;
use Drupal\webform\WebformSubmissionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Produces a Webform submission result containing the input data.
 *
 * @DataProducer(
 *   id = "webform_submit",
 *   name = @Translation("Webform submit"),
 *   description = @Translation("Create a webform submission."),
 *   produces = @ContextDefinition("any",
 *     label = @Translation("Webform submission result"),
 *     required = FALSE
 *   ),
 *   consumes = {
 *     "id" = @ContextDefinition("string",
 *       label = @Translation("Webform ID")
 *     ),
 *     "elements" = @ContextDefinition("any",
 *       label = @Translation("Array of WebformSubmissionElement objects.")
 *     ),
 *     "sourceEntityType" = @ContextDefinition("string",
 *       label = @Translation("Source entity type"),
 *       required = FALSE
 *     ),
 *     "sourceEntityId" = @ContextDefinition("string",
 *       label = @Translation("Source entity ID"),
 *       required = FALSE
 *     ),
 *   }
 * )
 */
class WebformSubmit extends DataProducerPluginBase implements ContainerFactoryPluginInterface {

  /**
   * Constructs a new WebformSubmit data producer.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $pluginId
   *   The plugin ID.
   * @param mixed $pluginDefinition
   *   The plugin definition.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\graphql\GraphQL\Utility\FileUpload $fileUpload
   *   GraphQL file upload service.
   * @param \Drupal\webform\Plugin\WebformElementManagerInterface $elementManager
   *   The Webform element manager service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory service.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    $pluginDefinition,
    protected readonly RendererInterface $renderer,
    protected readonly EntityTypeManagerInterface $entityTypeManager,
    protected readonly FileUpload $fileUpload,
    protected readonly WebformElementManagerInterface $elementManager,
    protected readonly ConfigFactoryInterface $configFactory,
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $pluginId, $pluginDefinition) {
    return new static(
      $configuration,
      $pluginId,
      $pluginDefinition,
      $container->get('renderer'),
      $container->get('entity_type.manager'),
      $container->get('graphql.file_upload'),
      $container->get('plugin.manager.webform.element'),
      $container->get('config.factory')
    );
  }

  /**
   * Resolves the Webform submission.
   *
   * @param string $id
   *   The Webform ID for which to create a submission.
   * @param array $elements
   *   Submitted values for Webform elements.
   * @param string|null $sourceEntityType
   *   The entity type of the entity that is the source of the submission.
   * @param string|null $sourceEntityId
   *   The ID of the entity that is the source of the submission.
   * @param \Drupal\graphql\GraphQL\Execution\FieldContext $field
   *   The GraphQL field context.
   *
   * @return \Drupal\graphql_webform\Model\WebformSubmissionResult
   *   The Webform submission result.
   */
  public function resolve(
    string $id,
    // The GraphQL field context is only populated if it is at the end of the
    // argument list. Suppress the coding standards warning for this.
    array $elements,
    ?string $sourceEntityType,
    ?string $sourceEntityId,
    FieldContext $field,
  ): WebformSubmissionResult {
    // Create context and execute validation and submission in render context to
    // capture caching metadata.
    $renderContext = new RenderContext();
    $renderResult = $this->renderer->executeInRenderContext($renderContext, function () use ($id, $elements, $field, $sourceEntityId, $sourceEntityType) {
      // Create the result object.
      $result = new WebformSubmissionResult();

      // Return an error if the requested Webform does not exist.
      $webform = $this->getWebformEntityStorage()->load($id);
      if (!$webform instanceof WebformInterface) {
        return $result->addError(sprintf('Webform %s does not exist.', $id));
      }

      $field->addCacheableDependency($webform);

      // Return an error if the Webform does not accept submissions.
      if (!$webform->isOpen()) {
        // Return the custom message defined in the form settings, falling back
        // to the default message. If the default message is empty, return a
        // helpful message.
        $message = $webform->getSetting('form_close_message');
        if (empty($message)) {
          $message = $this->getWebformSetting('settings.default_form_close_message');
        }
        if (empty($message)) {
          $message = 'This form is closed to new submissions.';
        }

        return $result->addError($message);
      }

      // Return an error if user is not allowed to create submissions.
      if (!$webform->access('submission_create')) {
        $accessDenied = Webform::buildAccessDenied($webform);
        $markup = (string) $this->renderer->render($accessDenied);
        return $result->addError($markup);
      }

      // The data elements, e.g. all form values that are not managed files.
      $dataElements = [];
      foreach ($elements as $element) {
        $dataElements[$element['element']] = $element['value'];
      }

      // Build the form data object for the form submission.
      $formData = [
        'webform_id' => $id,
        'data' => $dataElements,
      ];

      // Only include the source entity type and ID if both are provided.
      if ($sourceEntityId && $sourceEntityType) {
        $formData['entity_id'] = $sourceEntityId;
        $formData['entity_type'] = $sourceEntityType;
      }

      // Let the Webform module handle the validation of the form values.
      $validationErrors = WebformSubmissionForm::validateFormValues($formData) ?? [];
      $this->handleFormErrors($validationErrors, $result);

      // Check if everything is valid so far. Do not create a submission if
      // there are any validation errors.
      if (!$result->isValid()) {
        return $result;
      }

      // Try to create a submission.
      $submission = WebformSubmissionForm::submitFormValues($formData);

      if ($submission instanceof WebformSubmissionInterface) {
        // The submission was created successfully. Return it.
        $result->setSubmission($submission);
      }
      elseif (!empty($submission)) {
        // An error happened during submission creation.
        $this->handleFormErrors($submission, $result);
      }

      return $result;
    });

    if (!$renderContext->isEmpty()) {
      $field->addCacheableDependency($renderContext->pop());
    }

    return $renderResult;
  }

  /**
   * Populates errors on the result as WebformSubmissionValidationError objects.
   *
   * @param array $errors
   *   The errors returned by the Webform module.
   * @param \Drupal\graphql_webform\Model\WebformSubmissionResult $result
   *   The result object to populate.
   */
  protected static function handleFormErrors(array $errors, WebformSubmissionResult $result): void {
    foreach ($errors as $element => $message) {
      // The Webform module returns error messages intended to be shown in a web
      // page. Strip HTML tags to make them suitable for general use.
      $message = strip_tags((string) $message);
      $result->addValidationError(new WebformSubmissionValidationError([$message], $element));
    }
  }

  /**
   * Handle possible file uploads.
   *
   * @param \Drupal\webform\WebformInterface $webform
   *   The Webform entity.
   * @param array $files
   *   The array of WebformSubmissionFile items.
   * @param \Drupal\graphql_webform\Model\WebformSubmissionResult $result
   *   The result object to populate with validation errors.
   *
   * @return array
   *   The mapped array of WebformSubmissionElement items.
   */
  protected function handleFileElements(WebformInterface $webform, array $files, WebformSubmissionResult $result): array {
    // Array of all the managed file element keys.
    $managedFileElements = array_keys($webform->getElementsManagedFiles());

    // Contains the mapped submission values. The original $files array
    // contains the actual UploadedFile object. The mapped item contains the
    // FID, if upload was successfull.
    $filesMapped = [];

    $elementsMultiple = [];
    $elementsSingle = [];

    $input = [];

    foreach ($files as $item) {
      $elementKey = $item['element'];

      // Check if the element is actually a managed file.
      if (!in_array($elementKey, $managedFileElements)) {
        $validation = new WebformSubmissionValidationError(['Given element ID is not a managed file element.'], $item['element']);
        $result->addValidationError($validation);
        continue;
      }

      // Get the configuration for the element.
      $configuration = $webform->getElement($item['element']);

      /** @var \Drupal\webform\Plugin\WebformElement\WebformManagedFileBase $element */
      $element = $this->elementManager->getElementInstance($configuration);

      // Get file validation and upload settings.
      $fileExtensions = $element->getElementProperty($configuration, 'file_extensions');
      $maxFilesize = $element->getElementProperty($configuration, 'max_filesize');
      $uriScheme = $element->getElementProperty($configuration, 'uri_scheme');
      $multiple = $element->getElementProperty($configuration, 'multiple');

      if ($multiple === FALSE || $multiple === NULL || $multiple === 1) {
        $elementsSingle[] = $elementKey;

        // Check if we already have one file for this element.
        if (!empty($filesMapped[$elementKey])) {
          $validation = new WebformSubmissionValidationError(['Only one file is allowed.'], $item['element']);
          $result->addValidationError($validation);
          continue;
        }
      }
      else {
        $elementsMultiple[] = $elementKey;
      }

      // Perform the upload.
      $upload = $this->fileUpload->saveFileUpload($item['file'], [
        'file_extensions' => $fileExtensions,
        'max_filesize' => $maxFilesize,
        'uri_scheme' => $uriScheme,
        'file_directory' => 'webform',
      ]);

      // Get the upload violations.
      $violations = $upload->getViolations();

      // Convert the upload violations to validation errors.
      if (!empty($violations)) {
        $validation = new WebformSubmissionValidationError([], $item['element']);
        foreach ($violations as $violation) {
          $validation->addMessage($violation['message']);
        }
        $result->addValidationError($validation);
        continue;
      }

      $file = $upload->getFileEntity();
      // If there is no file and no validation errors, add an unexpected error.
      if (empty($file) && $result->isValid()) {
        $validation = new WebformSubmissionValidationError(['Unexpected error while uploading file.'], $item['element']);
        $result->addValidationError($validation);
        continue;
      }

      // Upload and saving was successful. Add the mapped form item, with the
      // FID as the value.
      $filesMapped[$elementKey][] = $file->id();
    }

    foreach ($filesMapped as $elementKey => $fileIds) {
      // Element supports multiple uploads.
      if (in_array($elementKey, $elementsMultiple)) {
        foreach ($fileIds as $index => $id) {
          $input[] = [
            'element' => $elementKey . "[$index]",
            'value' => $id,
          ];
        }
      }
      else {
        $input[] = [
          'element' => $elementKey,
          'value' => $fileIds[0],
        ];
      }
    }

    return $input;
  }

  /**
   * Returns the webform entity storage.
   *
   * @return \Drupal\webform\WebformEntityStorageInterface
   *   The webform entity storage.
   */
  protected function getWebformEntityStorage(): WebformEntityStorageInterface {
    return $this->entityTypeManager->getStorage('webform');
  }

  /**
   * Returns the setting from the Webform module with the given key.
   *
   * @param string $key
   *   The key of the setting to return.
   *
   * @return mixed
   *   The setting value.
   */
  protected function getWebformSetting(string $key) {
    return $this->configFactory->get('webform.settings')->get($key);
  }

}
